{% load i18n %}
{% load humanize %}

<div class="tab-pane card active" id="tab-library">
    <div class="card-body">
        <table id="table-blueprints" style="width: 100%" class="table table-hover table-striped">
            <thead>
                <tr>
                    <th></th>
                    <th>{% translate "Type" %}</th>
                    <th><abbr title="{% translate 'Quantity' %}">{% translate "Qty" %}</abbr></th>
                    <th>{% translate "Owner" %}</th>
                    <th><span class="fas fa-gem pad-right"></span><abbr title="{% translate 'Material Efficiency' %}">
                        {% comment %}Translators: Abbreviation for "Material Efficiency"{% endcomment %}
                        {% translate 'ME' %}
                    </abbr></th>
                    <th><span class="fas fa-hourglass pad-right"></span><abbr title="{% translate 'Time Efficiency' %}">
                        {% comment %}Translators: Abbreviation for "Time Efficiency"{% endcomment %}
                        {% translate 'TE' %}
                    </abbr></th>
                    <th>{% translate "Original" %}</th>
                    <th>{% translate "Runs Remaining" %}</th>
                    <th></th>
                </tr>
            </thead>
        </table>
        <p class="text-muted">
            {% translate "Blueprints in use are highlighted in blue." %}
        </p>
    </div>
</div>
