/* buttons
------------------------------------------------------------------------------------- */
@media all {
    /* the squared ones ... */
    .btn-square {
        width: 45px;
        height: 45px;
    }

    .btn-sm.btn-square {
        width: 35px;
        height: 35px;
    }
}

/* FIX: AA .tr-group
   I really hate to use !important -.-
   But it is what it is ...
------------------------------------------------------------------------------------- */
form.inline {
  display: inline;
}

.description-row {
  padding-top: 4px;
  padding-bottom: 4px;
}

.modal-tab {
  min-height: 300px;
}

#modalTitle > img {
  margin-left: 10px;
  margin-right: 10px;
}

.pad-right {
  padding-right: 5px;
}

#blueprint-title {
  margin: 15px;
  margin-top: 0px;
}
#blueprint-title > img {
  margin: 5px;
}

@media all {
  /* other */
  .nowrap {
      white-space: nowrap;
  }
}
