{% extends 'allianceauth/base-bs5.html' %}

{% load i18n %}

{% block header_nav_brand %}
    <a class="nav-link text-white link-light link-underline link-underline-opacity-0" href="{% url 'blueprints:index' %}">{% translate "Blueprints" %}</a>
{% endblock %}

{% block header_nav_collapse_left %}
    {% include 'blueprints/partials/menu/navigation.html' %}
{% endblock header_nav_collapse_left %}

{% block header_nav_collapse_right %}
    {% include 'blueprints/partials/menu/administration.html' %}
{% endblock header_nav_collapse_right %}

{% block page_title %}
    {% translate "Blueprint Library" %}
{% endblock %}

{% block content %}
    <div class="aa-blueprints">
        <h1 class="page-header text-center">{% translate "Blueprint Library" %}</h1>

        {% block details %}{% endblock %}
    </div>
{% endblock %}
