{% load i18n %}
{% load humanize %}

<div class="tab-pane card" id="tab-manage-blueprints">
    <div class="card-body">
        <table id="table-manage-blueprints" style="width: 100%" class="table table-striped">
            <thead>
                <tr>
                    <th>{% translate "Name" %}</th>
                    <th>{% translate "Quantity" %}</th>
                    <th></th>
                </tr>
            </thead>
        </table>
    </div>
</div>
